package sun.security.krb5;

import sun.security.krb5.internal.Krb5;
import java.security.PrivilegedAction;
import sun.security.krb5.internal.CredentialsUtil;
import sun.security.krb5.internal.crypto.EType;
import java.util.Locale;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import sun.security.krb5.internal.KDCOptions;
import java.io.IOException;
import java.net.InetAddress;
import java.util.Date;
import sun.security.krb5.internal.ccache.CredentialsCache;
import sun.security.krb5.internal.AuthorizationData;
import sun.security.krb5.internal.HostAddresses;
import sun.security.krb5.internal.KerberosTime;
import sun.security.krb5.internal.TicketFlags;
import sun.security.krb5.internal.Ticket;

public class Credentials
{
    Ticket ticket;
    PrincipalName client;
    PrincipalName clientAlias;
    PrincipalName server;
    PrincipalName serverAlias;
    EncryptionKey key;
    TicketFlags flags;
    KerberosTime authTime;
    KerberosTime startTime;
    KerberosTime endTime;
    KerberosTime renewTill;
    HostAddresses cAddr;
    AuthorizationData authzData;
    private static boolean DEBUG;
    private static CredentialsCache cache;
    static boolean alreadyLoaded;
    private static boolean alreadyTried;
    private Credentials proxy;
    
    public Credentials getProxy() {
        return this.proxy;
    }
    
    public Credentials setProxy(Credentials proxy) {
        this.proxy = proxy;
        return this;
    }
    
    private static native Credentials acquireDefaultNativeCreds(int[] p0);
    
    public Credentials(Ticket ticket, PrincipalName principalName, PrincipalName principalName2, PrincipalName principalName3, PrincipalName principalName4, EncryptionKey encryptionKey, TicketFlags ticketFlags, KerberosTime kerberosTime, KerberosTime kerberosTime2, KerberosTime kerberosTime3, KerberosTime kerberosTime4, HostAddresses hostAddresses, AuthorizationData authzData) {
        this(ticket, principalName, principalName2, principalName3, principalName4, encryptionKey, ticketFlags, kerberosTime, kerberosTime2, kerberosTime3, kerberosTime4, hostAddresses);
        this.authzData = authzData;
    }
    
    public Credentials(Ticket ticket, PrincipalName client, PrincipalName clientAlias, PrincipalName server, PrincipalName serverAlias, EncryptionKey key, TicketFlags flags, KerberosTime authTime, KerberosTime startTime, KerberosTime endTime, KerberosTime renewTill, HostAddresses cAddr) {
        this.proxy = null;
        this.ticket = ticket;
        this.client = client;
        this.clientAlias = clientAlias;
        this.server = server;
        this.serverAlias = serverAlias;
        this.key = key;
        this.flags = flags;
        this.authTime = authTime;
        this.startTime = startTime;
        this.endTime = endTime;
        this.renewTill = renewTill;
        this.cAddr = cAddr;
    }
    
    public Credentials(byte[] data, String s, String s2, String s3, String s4, byte[] array, int n, boolean[] flags, Date time, Date time2, Date time3, Date time4, InetAddress[] array2) throws KrbException, IOException {
        this(new Ticket(data), new PrincipalName(s, 1), (s2 == null) ? null : new PrincipalName(s2, 1), new PrincipalName(s3, 2), (s4 == null) ? null : new PrincipalName(s4, 2), new EncryptionKey(n, array), (flags == null) ? null : new TicketFlags(flags), (time == null) ? null : new KerberosTime(time), (time2 == null) ? null : new KerberosTime(time2), (time3 == null) ? null : new KerberosTime(time3), (time4 == null) ? null : new KerberosTime(time4), null);
    }
    
    public final PrincipalName getClient() {
        return this.client;
    }
    
    public final PrincipalName getClientAlias() {
        return this.clientAlias;
    }
    
    public final PrincipalName getServer() {
        return this.server;
    }
    
    public final PrincipalName getServerAlias() {
        return this.serverAlias;
    }
    
    public final EncryptionKey getSessionKey() {
        return this.key;
    }
    
    public final Date getAuthTime() {
        if (this.authTime != null) {
            return this.authTime.toDate();
        }
        return null;
    }
    
    public final Date getStartTime() {
        if (this.startTime != null) {
            return this.startTime.toDate();
        }
        return null;
    }
    
    public final Date getEndTime() {
        if (this.endTime != null) {
            return this.endTime.toDate();
        }
        return null;
    }
    
    public final Date getRenewTill() {
        if (this.renewTill != null) {
            return this.renewTill.toDate();
        }
        return null;
    }
    
    public final boolean[] getFlags() {
        if (this.flags == null) {
            return null;
        }
        return this.flags.toBooleanArray();
    }
    
    public final InetAddress[] getClientAddresses() {
        if (this.cAddr == null) {
            return null;
        }
        return this.cAddr.getInetAddresses();
    }
    
    public final byte[] getEncoded() {
        byte[] asn1Encode = null;
        try {
            asn1Encode = this.ticket.asn1Encode();
        }
        catch (Asn1Exception x) {
            if (Credentials.DEBUG) {
                System.out.println(x);
            }
        }
        catch (IOException x2) {
            if (Credentials.DEBUG) {
                System.out.println(x2);
            }
        }
        return asn1Encode;
    }
    
    public boolean isForwardable() {
        return this.flags.get(1);
    }
    
    public boolean isRenewable() {
        return this.flags.get(8);
    }
    
    public Ticket getTicket() {
        return this.ticket;
    }
    
    public TicketFlags getTicketFlags() {
        return this.flags;
    }
    
    public AuthorizationData getAuthzData() {
        return this.authzData;
    }
    
    public boolean checkDelegate() {
        return this.flags.get(13);
    }
    
    public void resetDelegate() {
        this.flags.set(13, false);
    }
    
    public Credentials renew() throws KrbException, IOException {
        KDCOptions kdcOptions = new KDCOptions();
        kdcOptions.set(30, true);
        kdcOptions.set(8, true);
        return new KrbTgsReq(kdcOptions, this, this.server, this.serverAlias, null, null, null, null, this.cAddr, null, null, null).sendAndGetCreds();
    }
    
    public static Credentials acquireTGTFromCache(PrincipalName principalName, String cache) throws KrbException, IOException {
        if (cache == null) {
            String s = (String)AccessController.doPrivileged(new GetPropertyAction("os.name"));
            if (s.toUpperCase(Locale.ENGLISH).startsWith("WINDOWS") || s.toUpperCase(Locale.ENGLISH).contains("OS X")) {
                Credentials acquireDefaultCreds = acquireDefaultCreds();
                if (acquireDefaultCreds == null) {
                    if (Credentials.DEBUG) {
                        System.out.println(">>> Found no TGT's in LSA");
                    }
                    return null;
                }
                else if (principalName != null) {
                    if (acquireDefaultCreds.getClient().equals(principalName)) {
                        if (Credentials.DEBUG) {
                            System.out.println(">>> Obtained TGT from LSA: " + acquireDefaultCreds);
                            return acquireDefaultCreds;
                        }
                        return acquireDefaultCreds;
                    }
                    else {
                        if (Credentials.DEBUG) {
                            System.out.println(">>> LSA contains TGT for " + acquireDefaultCreds.getClient() + " not " + principalName);
                        }
                        return null;
                    }
                }
                else {
                    if (Credentials.DEBUG) {
                        System.out.println(">>> Obtained TGT from LSA: " + acquireDefaultCreds);
                        return acquireDefaultCreds;
                    }
                    return acquireDefaultCreds;
                }
            }
        }
        CredentialsCache instance = CredentialsCache.getInstance(principalName, cache);
        if (instance == null) {
            return null;
        }
        Credentials initialCreds = instance.getInitialCreds();
        if (initialCreds == null) {
            return null;
        }
        if (EType.isSupported(initialCreds.key.getEType())) {
            return initialCreds;
        }
        if (Credentials.DEBUG) {
            System.out.println(">>> unsupported key type found the default TGT: " + initialCreds.key.getEType());
        }
        return null;
    }
    
    public static synchronized Credentials acquireDefaultCreds() {
        Credentials acquireDefaultNativeCreds = null;
        if (Credentials.cache == null) {
            Credentials.cache = CredentialsCache.getInstance();
        }
        if (Credentials.cache != null) {
            Credentials initialCreds = Credentials.cache.getInitialCreds();
            if (initialCreds != null) {
                if (Credentials.DEBUG) {
                    System.out.println(">>> KrbCreds found the default ticket granting ticket in credential cache.");
                }
                if (EType.isSupported(initialCreds.key.getEType())) {
                    acquireDefaultNativeCreds = initialCreds;
                }
                else if (Credentials.DEBUG) {
                    System.out.println(">>> unsupported key type found the default TGT: " + initialCreds.key.getEType());
                }
            }
        }
        if (acquireDefaultNativeCreds == null) {
            if (!Credentials.alreadyTried) {
                try {
                    ensureLoaded();
                }
                catch (Exception ex) {
                    if (Credentials.DEBUG) {
                        System.out.println("Can not load credentials cache");
                        ex.printStackTrace();
                    }
                    Credentials.alreadyTried = true;
                }
            }
            if (Credentials.alreadyLoaded) {
                if (Credentials.DEBUG) {
                    System.out.println(">> Acquire default native Credentials");
                }
                try {
                    acquireDefaultNativeCreds = acquireDefaultNativeCreds(EType.getDefaults("default_tkt_enctypes"));
                }
                catch (KrbException ex2) {}
            }
        }
        return acquireDefaultNativeCreds;
    }
    
    public static Credentials acquireServiceCreds(String service, Credentials initCreds) throws KrbException, IOException {
        return CredentialsUtil.acquireServiceCreds(service, initCreds);
    }
    
    public static Credentials acquireS4U2selfCreds(PrincipalName user, Credentials middleTGT) throws KrbException, IOException {
        return CredentialsUtil.acquireS4U2selfCreds(user, middleTGT);
    }
    
    public static Credentials acquireS4U2proxyCreds(String s, Ticket ticket, PrincipalName principalName, Credentials credentials) throws KrbException, IOException {
        return CredentialsUtil.acquireS4U2proxyCreds(s, ticket, principalName, credentials);
    }
    
    public CredentialsCache getCache() {
        return Credentials.cache;
    }
    
    public static void printDebug(Credentials credentials) {
        System.out.println(">>> DEBUG: ----Credentials----");
        System.out.println("\tclient: " + credentials.client.toString());
        if (credentials.clientAlias != null) {
            System.out.println("\tclient alias: " + credentials.clientAlias.toString());
        }
        System.out.println("\tserver: " + credentials.server.toString());
        if (credentials.serverAlias != null) {
            System.out.println("\tserver alias: " + credentials.serverAlias.toString());
        }
        System.out.println("\tticket: sname: " + credentials.ticket.sname.toString());
        if (credentials.startTime != null) {
            System.out.println("\tstartTime: " + credentials.startTime.getTime());
        }
        System.out.println("\tendTime: " + credentials.endTime.getTime());
        System.out.println("        ----Credentials end----");
    }
    
    static void ensureLoaded() {
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                if (System.getProperty("os.name").contains("OS X")) {
                    System.loadLibrary("osxkrb5");
                }
                else {
                    System.loadLibrary("w2k_lsa_auth");
                }
                return null;
            }
        });
        Credentials.alreadyLoaded = true;
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer("Credentials:");
        sb.append("\n      client=").append(this.client);
        if (this.clientAlias != null) {
            sb.append("\n      clientAlias=").append(this.clientAlias);
        }
        sb.append("\n      server=").append(this.server);
        if (this.serverAlias != null) {
            sb.append("\n      serverAlias=").append(this.serverAlias);
        }
        if (this.authTime != null) {
            sb.append("\n    authTime=").append(this.authTime);
        }
        if (this.startTime != null) {
            sb.append("\n   startTime=").append(this.startTime);
        }
        sb.append("\n     endTime=").append(this.endTime);
        sb.append("\n   renewTill=").append(this.renewTill);
        sb.append("\n       flags=").append(this.flags);
        sb.append("\nEType (skey)=").append(this.key.getEType());
        sb.append("\n   (tkt key)=").append(this.ticket.encPart.eType);
        return sb.toString();
    }
    
    static {
        Credentials.DEBUG = Krb5.DEBUG;
        Credentials.alreadyLoaded = false;
        Credentials.alreadyTried = false;
    }
}
